import os
import datetime
import tarfile
import os.path
import logging
import shutil
import kraken.tools.Utils as Utils
import kraken.tools.TextUi as TextUi
import kraken.entrytypes.MetaData as MetaData
import kraken.Constants
import kraken.query.FilterEditor as FilterEditor
import kraken.entrytypes.Entry as Entry
import kraken.entrytypes.MergeEntries as MergeEntries

def description(system, runtime):
    return """
    Export and import information from various formats.
    """
        

def execute(system, runtime):
   commands = [ 
      "Export BibTeX file",
      "Backup the whole kraken",
      "Export selected entries",
      "Export metadata as RDF",
      "-",
      "Import kraken directory",
      "-",
      "Return to main menu"
   ]
   commandname = TextUi.inputMenu("Select the command to execute:", commands)
   if commandname == "Export BibTeX file":
       exportBibTeX(system, runtime)    
       return
   if commandname == "Backup the whole kraken":
       backupKraken(system, runtime)
       return
   if commandname == "Export metadata as RDF":
       exportRDF(system, runtime)
       return
   if commandname == "Export selected entries":
       exportSelectedEntries(system, runtime)
       return
   if commandname == "Import kraken directory":
       importKrakenDirectory(system, runtime)
       return
   ## ....
   if commandname == "Return to main menu":
       return
   print "Unrecognized choice" 
   return
   

#
#  Execute the command
#
def exportBibTeX(system, runtime):
    #print "Generating Bibtex..."
    metadata = MetaData.loadMetaData(system, runtime)
    descriptor = metadata.descriptors["BibTex"]
    bibtexFile = "KrakenBibtex.bib"
    bibtexFile = TextUi.inputString("Name of bibtex file to generate:", bibtexFile)    
    FILE = open(bibtexFile, "w+")
    bibtexEntryCount = 0
    for entry in metadata.entries:        
        bibFilename = descriptor.getNativeFileName(entry) 
        if not os.path.exists(bibFilename):
           continue
        FILE.write("% Generated from kraken entry " + bibFilename + "\n")    
        BIBFILE = open(bibFilename)
        FILE.write(BIBFILE.read())
        BIBFILE.close()
        FILE.write("\n\n")
        bibtexEntryCount = bibtexEntryCount + 1
    FILE.close()
    print "Generated " + str(bibtexEntryCount) + " entries in file " + bibtexFile    

def backupKraken(system, runtime):
    n = datetime.datetime.now()
    strmonth = ("0" + str(n.month))[-2:]
    strday = ("0" + str(n.day))[-2:]
    filename = "krakenbackup_" + str(n.year) + strmonth + strday + ".tgz"
    filename = TextUi.inputString("Backup file:", initvalue = filename)    
    TextUi.startLongCommand("Creating backup archive...\n")
    f = tarfile.open(filename,"w:gz")
    f.posix = False
    i = 0    
    for a in system.getProperty("krakenDirs"):
        print "   adding "+a        
        f.add(a, "krakendir"+str(i))
        i = i + 1
    f.close()
    TextUi.endLongCommand()    
    
def exportRDF(system, runtime):
    metadata = MetaData.loadMetaData(system, runtime)
    MetaData.generateRdf(system, runtime, metadata)
    return
    
def exportSelectedEntries(system, runtime):
    print "Exporting the selected entries"
    metadata = MetaData.loadMetaData(system, runtime)
    exportDir = TextUi.inputString("Export to directory")
    if not os.path.isdir(exportDir):
        os.mkdir(exportDir)
    flt = FilterEditor.createFilterWithVerify(system, metadata)
    md = flt.processMetaData(metadata)
    for entry in md.entries:
        sourceDir = entry.getAbsoluteDirName()
        destDir = os.path.join(exportDir,entry.getProperty("directoryName"))
        print "Copying " + sourceDir + " to " + destDir
        shutil.copytree(sourceDir, destDir)

        
def importKrakenDirectory(system, runtime):
    print "Importing an external kraken directory"
    metadata = MetaData.loadMetaData(system, runtime)
    # create a hashmap of identifiers. It assumes that the current kraken 
    # is consistent
    ids = {}
    for entry in metadata.entries:
        identifier = entry.getResourceURI()
        if identifier in ids:
            logging.critical("Duplicate entries for id =" + identifier)
            logging.critical("Entry 1" + entry.getProperty("title"))
            logging.critical("Entry 2" + ids[identifier].getProperty("title"))
        else:
            ids[identifier] = entry
    importDir = TextUi.inputString("Import from directory")    
    if not os.path.isdir(importDir):
        logging.critical(importDir + "is not a valid directory")
        return
    importPrefix = TextUi.inputString("Prefix the imported entries with", "imp_")
    importCommands = {}        
    for entryDir in os.listdir(importDir):
        importEntryDir = os.path.join(importDir, entryDir)
        if not os.path.isdir(importEntryDir):
            return
        entry = Entry.loadKraEntry(system, importDir, entryDir)
        if entry == None:
            logging.critical("Unsuccesful loading of entry from importDir " + importDir)
            continue
        entry.parentDirName = importDir
        identifier = entry.getResourceURI()
        print "Importing: " + str(entry)
        if entry.getResourceURI() in ids:
            primaryEntry = ids[identifier]
            primaryEntryDir = primaryEntry.getAbsoluteDirName()
            logging.critical("This identifier is already in the kraken =" + identifier)
            logging.critical("Title of import entry: " + entry.getProperty("title"))
            logging.critical("Title of kraken entry: " + ids[identifier].getProperty("title"))
            print "This will be solved later by a conflict resolution process. For the time being, do not import"
            MergeEntries.mergeEntries(primaryEntryDir,importEntryDir)
            continue
        importedEntryDir = os.path.join(system.getProperty("destinationDir"), importPrefix+entryDir)            
        if os.path.exists(importedEntryDir):
            logging.critical("Imported entry dir exists" + importedEntryDir + ". Skipping.")
            continue
        print "Copying " + entryDir + " to " + importedEntryDir
        importCommands[os.path.join(importDir, entryDir)] = importedEntryDir
    print "Importing " + str(len(importCommands.keys())) + " entries."
    for a in importCommands:
        print "Copying " + a + " to " + importCommands[a]
        shutil.copytree(a, importCommands[a])
    
