import time
import os.path
import logging
from types import *
import pickle

from rdflib.URIRef import URIRef
from rdflib.Literal import Literal
from rdflib.Namespace import Namespace
from kraken.entrytypes.EntryBase import EntryBase
from kraken.tools.UniqueIdGenerator import getUniqueId
import kraken.Constants
from kraken.entrytypes.TypicalNamespaces import NS_KRAKEN

class System(EntryBase):
    
  def __init__(self):
      EntryBase.__init__(self)
      # self.__class__.defaultMapping = Namespace("http://www.cs.ucf.edu/~lboloni/Programming/KrakenWeb/")
      self.__class__.defaultMapping = NS_KRAKEN      
      self.resourceURI = URIRef("System_" + getUniqueId())
      # initialize the fields, to set the types - this is needed for the time
      self.registerProperty("destinationDir", os.path.expanduser(kraken.Constants.DEFAULT_DESTINATION_DIR))
      self.registerProperty("krakenDirs", [self.getProperty("destinationDir")])
      self.registerProperty("linkDir", os.path.expanduser(kraken.Constants.DEFAULT_LINK_DIR))
      self.registerProperty("generatedEntryDir", os.path.expanduser(kraken.Constants.DEFAULT_GENERATED_ENTRY_DIR))
      self.registerProperty("tempDir", os.path.expanduser(kraken.Constants.DEFAULT_STAGING_DIR))
      self.registerProperty("filterDir", os.path.expanduser(kraken.Constants.DEFAULT_FILTER_DIR))
      self.registerProperty("htmlDir", os.path.expanduser(kraken.Constants.DEFAULT_HTML_DIR))
      self.registerProperty("systemDir", os.path.expanduser(kraken.Constants.DEFAULT_SYSTEM_DIR))
      
  #
  #  Command line based editing of an entry - mainly for annotation, etc
  #            
  def edit(self, metadata = None):
      print "System configuration file"
      self.editField("destinationDir")
      self.editField("krakenDirs")
      self.editField("linkDir")
      self.editField("generatedEntryDir")
      self.editField("tempDir")
      self.editField("htmlDir")
      self.editField("systemDir")

      
      
      
#
# creates the config file, either by loading it, or creating a new one
# 
def createSystem():
    # try loading from the rdf
    sysfile = os.path.expanduser(kraken.Constants.SYSTEM_FILE)
    sys = System()
    if os.path.exists(sysfile):
        print "Loading config from system file"
        sys.readEntryFromRdf(sysfile)
    else:
        logging.info("Could not find system file:" + sysfile + ". Going with defaults.")
    return sys


      
