from kraken.entrytypes.MetaData import MetaData
import string
import copy

def compare(a, b, ranker):
    # print ranker
    return ranker.compare(a, b)
    
class Sorter:
    def __init__(self):
        pass
       
    def compare(self, a, b):
        return 1

    def processMetaData(self, metadata):
       md = copy.deepcopy(metadata)
       if self.__class__.__name__ != "NoRanking":    
          md.entries.sort(lambda a, b: compare(a, b, self))
       else:
          print "No ranking needed!"
       return md
        
        
    def __str__(self):
        return ""

class NoSorting(Sorter):
    pass
            
        
class ReverseSorter(Sorter):
    def __init__(self):
        self.theSorter = None
        
    def compare(self, a, b):
        return self.theSorter.compare(b,a)
        
    def __str__(self):
        return "(SORT-REVERSE " + str(self.theSorter) + ")" 
        
        
class MultiLevelSorter(Sorter):
    """
      A sorter which is composed of a set of other sorters, which are evaluated
      in order. If one is 0 then we are going to the next one. 
    """
    def __init__(self):
        self.sorters = []
        
    def compare(self, a, b):
        for srt in self.sorters:
            val = srt.compare(a, b)
            if val != 0:
                return val
        return val
        
class EvaluationSorter(Sorter):
    def __init__(self):
        self.propertyName = None
        self.ignoreCase = 0
        
    def compare(self, a, b):        
        aValue = a.getExtendedProperty(self.propertyName)
        bValue = b.getExtendedProperty(self.propertyName)
        if aValue == None: 
            return -1
        elif bValue == None:
            return 1
        if not self.ignoreCase:
           return cmp(aValue, bValue)
        else:
           aValue = string.lower(str(aValue))
           bValue = string.lower(str(bValue))
           return cmp(aValue, bValue)

    def __str__(self):
        if not self.ignoreCase:
            return "(SORT-BY-PROPERTY " + self.propertyName + ")"
        else:
            return "(SORT-BY-PROPERTY-IGNORE-CASE " + self.propertyName + ")"
              
        
class FallbackSorter(Sorter):
    '''
       Take the sorters one by one. If one is equal, pick the next one...
    '''        
    def __init__(self):
        self.sorters = []
        
    def compare(self, a, b):
        for srtr in sorters:
            val = srtr.compare(a,b)
            if val != 0:
                return val
            # else, go to the next one.     
        return 1
        
class PreferredValueSorter(Sorter):
    '''
       Preferred value sorter - some preferred values for enumeration type variables...
       Or for sequences
    '''        
    def __init__(self):
        self.propertyName = None
        self.preferredValues = []
        self.notPreferredValues = []
        
    def compare(self, a, b):
        aValue = a.getExtendedProperty(self.propertyName)
        bValue = b.getExtendedProperty(self.propertyName)
        if a == b:
            return 0
        for val in self.preferredValues:
            if val in aValue:
                return 1
            if val in bValue:
                return -1
        for val in self.notPreferredValues:
            if val in aValue:
                return -1
            if val in bValue:
                return 1
        return 0
        
